(*
 * Graph: generic graph library
 * Copyright (C) 2004
 * Sylvain Conchon, Jean-Christophe Filliatre and Julien Signoles
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(* $Id: rand.ml,v 1.1 2004/02/03 21:42:25 filliatr Exp $ *)

module type S = sig
  type graph 
  val random : v:int -> e:int -> graph
end

module Generic(B : Builder.INT) = struct

  type graph = B.G.t

  open B

  let fold_for i0 i1 f =
    let rec loop i v = if i > i1 then v else loop (i + 1) (f v i) in
    loop i0

  let random ~v ~e =
    let a = Array.init v G.V.create in
    let g = Array.fold_left add_vertex (empty ()) a in
    fold_for 1 e (fun g i -> add_edge g a.(Random.int v) a.(Random.int v)) g

end

module P (G : Sig.P with type V.label = int) = Generic(Builder.P(G))

module I (G : Sig.I with type V.label = int) = Generic(Builder.I(G))
